@@ -129,6 +129,18 @@ True
 >>> bound[''] is bound
 True
 
+To get at the underlying context, use the .context attribute.  This may
+create lookup the current context if necessary.  This property also exists
+on contexts as well as ref so it can be used ubiquitously if needed.
+
+>>> ctx = BWContext('root').throw()
+>>> ctx.context is ctx                  # Context self-ref
+True
+>>> ctx['hello'].context is ctx         # Bound context ref
+True
+>>> BWContext['hello'].context          # Unbound context ref
+<root{1}>
+
 ========================================
 === Dereferencing Context References ===
 ========================================
@@ -268,7 +280,7 @@ this is equivalent to the above "bottom" case:
 Contexts can are BWThrowables (see bwthrowable) and can be retrieved via
 the class property BWContext.CURRENT:
 
->>> rootctx.throw()
+>>> ctx = rootctx.throw()
 >>> BWContext.CURRENT is rootctx
 True
 
@@ -328,6 +340,85 @@ declarations:
 >>> print -ctx.user
 None
 
+======================
+=== Dynamic values ===
+======================
+
+Objects placed in a context that define a __ctxproperty__ attribut will
+have that attribute called (as a function or method) upon access by a
+context.  The property function will receive the context, the subpath, and
+a default value and is responsible for determining the correct value given
+those arguments or returning default if inappropriate or not found.
+
+The BWContext .property() method will convert a normal Python function into
+an object that produces the described effect.
+
+>>> def printer(ctx, subpath, default):
+...     return 'Hello %s' % subpath
+...
+>>> ctx = BWContext('test-property')
+>>> ctx['hello.'] = ctx.property(printer)
+>>> ctx.hello.world
+<test-property/hello.world => 'Hello world'>
+
+This could be defined in objects as the following:
+
+>>> from bullwinkle import after_super, before_super
+>>> class MyPrinter(BWContextInstallable, BWContextProperty):
+...     prefix = member(str, default='Hello')
+...     exclude = member(str, None, default=None)
+...
+...     @before_super
+...     def ctx_access(self, ctx, subpath, default):
+...         if subpath != self.exclude:
+...             return '%s %s!' % (self.prefix, subpath)
+...
+...     @after_super
+...     def ctx_install(self, ctx):
+...         ctx[''] = self      # All accesses below this go to self too.
+...
+>>> ctx = BWContext('test-property-object')
+>>> ctx.hello = MyPrinter()
+>>> ctx.hello.world
+<test-property-object/hello.world => 'Hello world!'>
+
+Contrast this with:
+
+>>> ctx = BWContext('test-property-object')
+>>> ctx.hello = MyPrinter(exclude='world')
+>>> ctx.hello.world
+<test-property-object/hello.world => :missing:>
+
+==========================
+=== Installing Objects ===
+==========================
+
+Objects that define an __installctx__ method will have that method called
+when the object is assigned to a context.   This method will receive a
+context ref indiciating where the object should be installed.  The
+installation occurs on a temporary sub-context which is then merged if the
+installation is successful.  Additionally, any variables set in the process
+are tracked and removed if del is later invoked on the path.
+
+As noted in the example of the previous section, the BWContextInstallable
+base class will call 'ctx_install' instead, making the code slightly more
+readable.
+
+Installers can also use the context's "property" method to define property
+functions:
+
+>>> class HelloMachine(BWContextInstallable):
+...     def ctx_install(self, ctx):
+...         ctx[''] = ctx.property(lambda c, s, d: self.helloizer(s))
+...
+...     def helloizer(self, subpath):
+...         return 'Hello %s' % subpath
+...
+>>> ctx = BWContext('test-install')
+>>> ctx.hello = HelloMachine()
+>>> ctx.hello.world
+<test-install/hello.world => 'Hello world'>
+
 =======================
 === Dynamic getters ===
 =======================
@@ -362,6 +453,8 @@ implementation will either:
 >>> sub = BWContext(ctx)
 >>> sub.whole_world.something
 <test{1}/whole_world.something => 'hello world'>
+>>> sub.something_else
+<test{1}/something_else => :missing:>
 
 Note that the first example is :missing: because the test is for things
 that start with "whole_world.", not just "whole_world".
@@ -545,7 +638,7 @@ class BWContext(BWThrowable):
                             self=self, varkeys=varkeys, storage=storage):
                 obj = storage.get(key, NOT_FOUND)
                 subkey = None
-                if obj is NOT_FOUND:
+                if obj is NOT_FOUND and isinstance(key, basestring):
                     # We've already tested for at least one varkey.
                     for prefix, value in varkeys: # pragma: no partial
                         if key.startswith(prefix):
@@ -554,13 +647,19 @@ class BWContext(BWThrowable):
                             break
                     else:
                         obj = default
-                if getattr(obj, '__ctxproperty__', False):
-                    obj = obj.__ctxproperty__(self, subkey, default)
-                return obj
+                return self._getprop(obj, default, subkey)
             return getter
         else:
             return storage.get
 
+    def _getprop(self, obj, default=None, subkey=None):
+        propfn = getattr(obj, '__ctxproperty__', None)
+        if propfn is True:
+            obj = obj(self, subkey, default)
+        elif propfn is not None:
+            obj = obj.__ctxproperty__(self, subkey, default)
+        return obj
+
     @cached
     def _bro(self):
         bases = self._basectx
@@ -603,9 +702,7 @@ class BWContext(BWThrowable):
             obj = default
         if obj is DELETED:
             obj = default
-        if getattr(obj, '__ctxproperty__', False):
-            obj = obj(self, None, default)
-        return obj
+        return self._getprop(obj, default)
 
     def __getattr__(self, name, NOT_FOUND=NOT_FOUND):
         if name.startswith('_'):
@@ -623,28 +720,47 @@ class BWContext(BWThrowable):
         return BWBoundContextRef(self, key.replace('__', '.'))
 
     def __setitem__(self, key, value, NOT_FOUND=NOT_FOUND):
+        uninstall = self.get((key, '__uninstall__'), None)
+        if uninstall is not None:
+            self._storage.update(uninstall)
+
         fn = getattr(value, '__installctx__', None)
-        if fn is not None:
+        if (fn is not None and
+            self.get(('__installed__', key, id(value))) is None):
+
             del self[key]
-            install_ctx = type(self)(None, self)
+            install_ctx = self()
+            install_ctx._storage['__installed__', key, id(value)] = True
             fn(install_ctx[key])
             uninstall = {}
             for ikey in install_ctx._ref:
                 uninstall[ikey] = self.get(ikey, DELETED)
             self._storage.update(install_ctx._ref)
-            self._storage[key, 'uninstall'] = uninstall
+            if install_ctx._varkeys:
+                self.__dict__['_varkeys'] = \
+                    (self._varkeys + install_ctx._varkeys)
+            self._storage[key, '__uninstall__'] = uninstall
+            self.__dict__.pop('_storage_getfn', None)
+            self.__dict__.pop('_getters', None)
+
         if isinstance(key, basestring) and key.endswith('.'):
             self.__dict__['_varkeys'] = self._varkeys + ((key, value),)
             self._storage[key[:-1]] = value
+            self.__dict__.pop('_storage_getfn', None)
+            self.__dict__.pop('_getters', None)
         else:
             self._storage[key] = value
 
     def __delitem__(self, key):
-        uninstall = self.get((key, 'uninstall'), None)
+        uninstall = self.get((key, '__uninstall__'), None)
         if uninstall:
             self._storage.update(uninstall)
         self._storage[key] = DELETED
 
+    @property
+    def context(self):
+        return self
+
     def property(self, fn):
         '''
         Declares the function to be a context property (it will be called
@@ -665,6 +781,20 @@ class BWContext(BWThrowable):
     def __ctxproperty__(self, basectx, subpath, default=None):
         return self.get(subpath, default)
 
+class BWContextInstallable(BWObject):
+    def __installctx__(self, ctx):
+        self.ctx_install(ctx)
+
+    def ctx_install(self, ctx):
+        pass
+
+class BWContextProperty(BWObject):
+    def __ctxproperty__(self, ctx, subpath, default):
+        return self.ctx_access(ctx, subpath, default)
+
+    def ctx_access(self, ctx, subpath, default):
+        return default
+
 class BWContextRef(BWObject):
     def __pos__(self, NOT_FOUND=NOT_FOUND):
         obj = self._self(NOT_FOUND)
@@ -728,7 +858,9 @@ class BWBoundContextRef(BWContextRef):
             return self
 
     def __setitem__(self, subpath, value):
-        self._ctx[self._path + '.' + subpath] = value
+        # Prevent recursion on ctx[''] = installable
+        if subpath or not self._path.endswith('.'):
+            self._ctx[self._path + '.' + subpath] = value
 
     def __delitem__(self, subpath):
         del self._ctx[self._path + '.' + subpath]
@@ -746,6 +878,10 @@ class BWBoundContextRef(BWContextRef):
     def _ctxtype(self):
         return type(self._ctx)
 
+    @cached
+    def context(self):
+        return self._ctx
+
 class BWUnboundContextRef(BWContextRef):
     def __init__(self, ctxtype, path):
         self.__dict__['_ctxtype'] = ctxtype
@@ -764,3 +900,7 @@ class BWUnboundContextRef(BWContextRef):
     def __repr__(self):
         return '<*%s/%s>' % (self._ctxtype.__name__, self._path)
 
+    @cached
+    def context(self):
+        return self._ctxtype(self._ctxtype.CURRENT)
+
